@@ -15,6 +15,16 @@ int js_isstringobject(js_State *J, int idx)
 	return js_isobject(J, idx) && js_toobject(J, idx)->type == JS_CSTRING;
 }
 
+int js_isbooleanobject(js_State *J, int idx)
+{
+	return js_isobject(J, idx) && js_toobject(J, idx)->type == JS_CBOOLEAN;
+}
+
+int js_isdateobject(js_State *J, int idx)
+{
+	return js_isobject(J, idx) && js_toobject(J, idx)->type == JS_CDATE;
+}
+
 static void jsonnext(js_State *J)
 {
 	J->lookahead = jsY_lexjson(J);
@@ -362,7 +372,9 @@ static void JSON_stringify(js_State *J)
 {
 	js_Buffer *sb = NULL;
 	char buf[12];
-	const char *s, *gap;
+	/* NOTE: volatile to silence GCC warning about longjmp clobbering a variable */
+	const char * volatile gap;
+	const char *s;
 	int n;
 
 	gap = NULL;
